#version 110

uniform sampler2D renderSampler;
uniform sampler2D randomSampler;
uniform sampler2D depthSampler;
uniform float bloomAlpha;
uniform float texAspect;
uniform vec2 maximums;
uniform vec2 units;
uniform float zNear;
uniform float zFar;

uniform vec2 samples1[9];
uniform vec2 samples2[9];

float transformDepth(float z_b);

void main()
{
	vec2 texCoord = vec2(gl_TexCoord[0]);
   	float uzCenter = texture2D(depthSampler, texCoord).r;
   	
   	vec2 len = vec2(0.0025*uzCenter)*vec2(1.0, texAspect);
   	
   	float z_center = transformDepth(uzCenter);
   	float peak = max(0.0005, 0.075*z_center);
   	float zmul = max(0.0,1.0-pow(z_center,0.33))/12.5;
    
    float z_sum = 0.0;
    vec3 colorSum = vec3(0.0);    
    
    vec2 random1 = normalize(texture2D(randomSampler, gl_FragCoord.xy/vec2(4.0)).xy - vec2(0.5));
    #ifndef NO_BLOOM
    vec2 random2 = normalize(texture2D(randomSampler, gl_FragCoord.xy/vec2(2.0)).xy - vec2(0.5));
    #endif

    int index;
	for(index = 0; index < 9; index += 1)
    {
        #ifndef NO_BLOOM
	    vec2 offset2 = len*reflect(samples2[index], random2);
	    colorSum += texture2D(renderSampler, min(texCoord+offset2, maximums)).rgb;
        #endif
    	vec2 offset1 = len*reflect(samples1[index], random1); 
	    float zA = transformDepth(texture2D(depthSampler, min(texCoord+offset1, maximums)).r);
	    float zB = transformDepth(texture2D(depthSampler, min(texCoord-offset1, maximums)).r);
	    
	    float diff = 1.0-abs(2.0*z_center-zA-zB-peak)/peak;
	    z_sum += clamp(diff, -z_center*4.0, 0.5);
	}
    #ifndef NO_BLOOM
    colorSum = colorSum*0.35*bloomAlpha;
    #endif
	
	vec3 origColor = texture2D(renderSampler, texCoord).rgb;
    float val = min(dot(origColor, vec3(1.0, 2.0, 1.0)), 3.0);
     
    gl_FragColor.rgb = vec3(0.5 + colorSum - (origColor + colorSum) * z_sum * zmul * val);
    gl_FragColor.a = gl_Color.a;
}

float transformDepth(float z_b)
{
	z_b = 2.0 * z_b - 1.0;
	//Was 2*zNear*zFar / (etc). Divided by zFar to get (very close) approximate result in range (0-1).
	return 2.0 * zNear / (zFar + zNear - z_b * (zFar - zNear)) / 0.055;
}